/*
I : a intensidade da componente em um canal de cor.
M : refletncia do material (d para difusa, s para especular).
R : transmitncia da luz (d para difusa, s para especular).
N : vetor normal da superfcie
L : vetor da direo da luz a partir do vrtice
R : vetor L refletido por N
V : vetor da direo do observador a partir do vrtice
H : vetor bissetriz entre L e V
r: rugosidade do material
s: brilhosidade do material
*/

uniform bool wardEnabled;
uniform bool phongEnabled;
uniform bool ashikhmin_shirleyEnabled;
uniform float r;

varying vec3 N;
varying vec3 V;
varying vec3 L;

vec4 specularWard( vec4 Ms, float NdotL, float NdotH, vec3 N, vec3 H, float r, float NdotV ) {
	return Ms * NdotL * exp(- pow(tan(acos(NdotH))/r, 2.0)) / ( 4.0*3.14*pow(r,2.0) * sqrt(NdotL*NdotV) );
}

vec4 specularPhong( vec4 Ms, float NdotH, float s, vec4 Rs ) {
	return Ms * Rs * pow( NdotH, s );
}

vec4 diffuseAshikhminShirley( vec4 Md, vec4 Rd, vec4 Ms, float NdotL, float NdotV ) {
	return 28.0 * Md * Rd / (23.0 * 3.14) * (1.0 - Ms) * (1.0 - pow((1.0 - NdotL/2.0), 5.0)) * (1.0 - pow((1.0 - NdotV/2.0), 5.0));
}

// N normalized vertex normal in eye space.
// V vertex position in eye space.
//
vec4 lighting( void )
{
	vec3  H = normalize( L + V );

	float NdotL = dot( N,L );
	float NdotV = dot( N,V );
	float NdotH = dot( N,H );

	vec4 Ms = gl_FrontMaterial.specular;
	vec4 Md = gl_FrontMaterial.diffuse;
	float s = gl_FrontMaterial.shininess;

	vec4 Rs = gl_LightSource[0].specular;
	vec4 Rd = gl_LightSource[0].diffuse;

	vec4 result = vec4(0, 0, 0, 0);
	if (wardEnabled) {
		result += specularWard( Ms, NdotL, NdotH, N, H, r, NdotV );;
	}
	if (phongEnabled) {
		result += specularPhong( Ms, NdotH, s, Rs );;
	}
	if (ashikhmin_shirleyEnabled) {
		result += diffuseAshikhminShirley( Md, Rd, Ms, NdotL, NdotV );;
	}
	return result ;
}

void main( void )
{
	gl_FragColor = lighting();
}


